@@ -3,12 +3,29 @@ package POE::Component::Server::TCP;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.370'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Socket qw(INADDR_ANY inet_ntoa inet_aton AF_INET AF_UNIX PF_UNIX);
 use Errno qw(ECONNABORTED ECONNRESET);
 
+BEGIN {
+  # under perl-5.6.2 the warning "leaks" from the eval, while newer versions don't...
+  # it's due to Exporter.pm behaving differently, so we have to shut it up
+  no warnings 'redefine';
+  local *Carp::carp = sub { die @_ };
+
+  # Socket::GetAddrInfo provides getaddrinfo where earlier Perls' Socket don't.
+  eval { Socket->import('getaddrinfo') };
+  if ($@) {
+    # :newapi is legacy, but we include it to be sure in case the user has an old version of GAI
+    eval { require Socket::GetAddrInfo; Socket::GetAddrInfo->import( qw(:newapi getaddrinfo) ) };
+    if ($@) {
+      *getaddrinfo = sub { Carp::confess("Unable to use IPv6: Socket::GetAddrInfo not available") };
+    }
+  }
+}
+
 # Explicit use to import the parameter constants.
 use POE::Session;
 use POE::Driver::SysRW;
@@ -99,6 +116,7 @@ sub new {
   my $session_type        = delete $param{SessionType};
   my $session_params      = delete $param{SessionParams};
   my $server_started      = delete $param{Started};
+  my $server_stopped      = delete $param{Stopped};
   my $listener_args       = delete $param{ListenerArgs};
 
   $listener_args = [] unless defined $listener_args;
@@ -117,6 +135,16 @@ sub new {
 
   my $client_args = delete($param{ClientArgs}) || delete($param{Args});
 
+  if ( (defined $client_infilter and ! defined $client_outfilter) or
+    (defined $client_outfilter and ! defined $client_infilter) ) {
+    croak "ClientInputFilter must be used with ClientOutputFilter";
+  }
+
+  if (defined $client_filter and defined $client_infilter) {
+    carp "ClientFilter ignored with ClientInputFilter and ClientOutputFilter";
+    undef $client_filter;
+  }
+
   # Defaults.
 
   $concurrency = -1 unless defined $concurrency;
@@ -264,8 +292,7 @@ sub new {
                 $heap->{remote_ip} = inet_ntoa($remote_addr);
               }
               else {
-                $heap->{remote_ip} =
-                  Socket6::inet_ntop($domain, $remote_addr);
+                $heap->{remote_ip} = ( getaddrinfo($remote_addr) )[1];
               }
 
               $heap->{remote_port} = $remote_port;
@@ -273,9 +300,11 @@ sub new {
               my $socket = $_[ARG0];
               if ($client_pre_connect) {
                 $socket = $client_pre_connect->(@_);
-                unless (fileno($socket)) {
-                  # TODO - Error callback?  Disconnected callback?
-                  # TODO - Should we do this before starting the child?
+                unless (defined($socket) and ref($socket) and fileno($socket)) {
+                  # TODO - The user ought to know what's going on
+                  # here, since it's triggered by something their
+                  # callback has done.  Should we expose a callback
+                  # anyway to avoid potential confusion?
                   return;
                 }
               }
@@ -454,7 +483,7 @@ sub new {
       # We accepted a connection.  Do something with it.
       tcp_server_got_connection => $accept_callback,
 
-      # conncurrency on close.
+      # concurrency on close.
       disconnected => sub {
         $_[HEAP]->{connections}--;
         DEBUG and warn(
@@ -513,6 +542,7 @@ sub new {
       # Dummy states to prevent warnings.
       _stop   => sub {
         DEBUG and warn "$$: $_[HEAP]->{alias} _stop";
+        $server_stopped and $server_stopped->(@_);
         undef($accept_session_id);
         return 0;
       },
@@ -533,14 +563,9 @@ sub _get_filters {
         "InputFilter"  => _load_filter($client_infilter),
         "OutputFilter" => _load_filter($client_outfilter)
       );
-      if (defined $client_filter) {
-        carp(
-          "ClientFilter ignored with ClientInputFilter or ClientOutputFilter"
-        );
-      }
     }
     elsif (defined $client_filter) {
-     return ( "Filter" => _load_filter($client_filter) );
+      return ( "Filter" => _load_filter($client_filter) );
     }
     else {
       return ( Filter => POE::Filter::Line->new(), );
@@ -703,6 +728,10 @@ set to a copy of the values in the server's C<ListenerArgs>
 constructor parameter.  The other parameters are standard for
 POE::Session's _start handlers.
 
+The component's C<Stopped> callback is invoked at the beginning of the
+master session's _stop routine. The parameters are standard for
+POE::Session's _stop handlers.
+
 The component's C<Error> callback is invoked when the server has a
 problem listening for connections.  C<Error> may also be called if the
 component's default acceptor has trouble accepting a connection.
@@ -723,6 +752,7 @@ connection---and not in the master listening session.  This has been a
 major point of confusion.  We welcome suggestions for making this
 clearer.
 
+=for comment
 TODO - Document some of the implications of having each connection
 handled by a separate session.
 
@@ -741,15 +771,21 @@ included in C<ClientConnected>'s parameters as @_[ARG0..$#_].
     ...
   }
 
-C<ClientPreConnect> is called before C<ClientConnected>, and it has
-different parameters: $_[ARG0] contains a copy of the client socket
-before it's given to POE::Wheel::ReadWrite for management.  Most HEAP
-members are set, except of course $_[HEAP]{client}, because the
-POE::Wheel::ReadWrite has not yet been created yet.
-C<ClientPreConnect> may enable SSL on the socket, using
-POE::Component::SSLify.  C<ClientPreConnect> must return a valid
-socket to complete the connection; the client will be disconnected if
-anything else is returned.
+C<ClientPreConnect> is called before C<ClientConnected>, and its
+purpose is to allow programs to reject connections or condition
+sockets before they're given to POE::Wheel::ReadWrite for management.
+
+The C<ClientPreConnect> handler is called with the client socket in
+$_[ARG0], and its return value is significant.  It must return a
+valid client socket if the connection is acceptable.  It must return
+undef to reject the connection.
+
+Most $_[HEAP] values are valid in the C<ClientPreConnect> handler.
+Obviously, $_[HEAP]{client} is not because that wheel hasn't been
+created yet.
+
+In the following example, the C<ClientPreConnect> handler returns the
+client socket after it has been upgraded to an SSL connection.
 
   sub handle_client_pre_connect {
 
@@ -822,11 +858,12 @@ session that will be listening for new connections.
 Many of the constructor parameters have been previously described.
 They are covered briefly again below.
 
-=head3 Server Sesson Configuration
+=head3 Server Session Configuration
 
 These constructor parameters affect POE::Component::Server::TCP's main
 listening session.
 
+=for comment
 TODO - Document the shutdown procedure somewhere.
 
 =head4 Acceptor
@@ -864,6 +901,7 @@ supports.  At the time of this writing, that may be a dotted IPv4
 quad, an IPv6 address, a host name, or a packed Internet address.  See
 also L</Hostname>.
 
+=for comment
 TODO - Example, using the lines below.
 
   Address => '127.0.0.1'   # Localhost IPv4
@@ -931,8 +969,10 @@ C<Domain> sets the address or protocol family within which to operate.
 The C<Domain> may be any value that POE::Wheel::SocketFactory
 supports.  AF_INET (Internet address space) is used by default.
 
-Use AF_INET6 for IPv6 support.  This constant is exported by Socket6,
-which must be loaded B<before> POE::Component::Server::TCP.
+Use AF_INET6 for IPv6 support.  This constant is exported by L<Socket>
+or L<Socket6>, depending on your version of Perl. Also be sure to have
+L<Socket::GetAddrInfo> installed, which is required by
+L<POE::Wheel::SocketFactory> for IPv6 support.
 
 =head4 Error
 
@@ -943,8 +983,7 @@ parameters as discussed there.
 
 A default error handler will be provided if Error is omitted.  The
 default handler will log the error to STDERR and shut down the server.
-Active connections will be permitted to to complete their
-transactions.
+Active connections will be permitted to complete their transactions.
 
   Error => sub {
     my ($syscall_name, $err_num, $err_str) = @_[ARG0..ARG2];
@@ -975,7 +1014,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 ObjectStates
 
@@ -989,7 +1029,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 PackageStates
 
@@ -1004,7 +1045,8 @@ will not reach these handlers.
 
 If POE::Kernel::ASSERT_USAGE is enabled, the constructor will croak() if it
 detects a state that it uses internally. For example, please use the "Started"
-callback if you want to specify your own "_start" event.
+and "Stopped" callbacks if you want to specify your own "_start" and "_stop"
+events respectively.
 
 =head4 Port
 
@@ -1023,10 +1065,17 @@ main server session's context.  It notifies the server that it has
 fully started.  The callback's parameters are the usual for a
 session's _start handler.
 
+=head4 Stopped
+
+C<Stopped> sets an optional callback that will be invoked within the
+main server session's context.  It notifies the server that it has
+fully stopped.  The callback's parameters are the usual for a
+session's _stop handler.
+
 =head4 ListenerArgs
 
 C<ListenerArgs> is passed to the listener session as the C<args> parameter.  In
-other words, it must be an arrayref, and the values are are passed into the
+other words, it must be an arrayref, and the values are passed into the
 C<Started> handler as ARG0, ARG1, etc.
 
 =head3 Connection Session Configuration
@@ -1121,7 +1170,9 @@ given object.
   ClientFilter => POE::Filter::Line->new(Literal => "\n"),
 
 C<ClientFilter> is optional.  The component will use
-"POE::Filter::Line" if it is omitted.
+"POE::Filter::Line" if it is omitted.  There is L</ClientInputFilter>
+and L</ClientOutputFilter> if you want to specify a different filter
+for both directions.
 
 Filter modules are not automatically loaded.  Be sure that the program
 loads the class before using it.
@@ -1172,7 +1223,8 @@ prohibits the other.
 
 C<ClientInputFilter> is used with C<ClientOutputFilter> to specify
 different protocols for input and output.  Both must be used together.
-Both follow the same usage as L</ClientFilter>.
+Both follow the same usage as L</ClientFilter>.  Overrides the filter set
+by L</ClientFilter>.
 
   ClientInputFilter  => [ "POE::Filter::Line", Literal => "\n" ],
   ClientOutputFilter => 'POE::Filter::Stream',
@@ -1181,7 +1233,8 @@ Both follow the same usage as L</ClientFilter>.
 
 C<ClientOutputFilter> is used with C<ClientInputFilter> to specify
 different protocols for input and output.  Both must be used together.
-Both follow the same usage as L</ClientFilter>.
+Both follow the same usage as L</ClientFilter>.  Overrides the filter set
+by L</ClientFilter>.
 
   ClientInputFilter  => POE::Filter::Line->new(Literal => "\n"),
   ClientOutputFilter => 'POE::Filter::Stream',
@@ -1383,11 +1436,12 @@ Some use cases require different session classes for the listener and
 the connection handlers.  This isn't currently supported.  Please send
 patches. :)
 
+=for comment
 TODO - Document that Reuse is set implicitly.
 
 =head1 AUTHORS & COPYRIGHTS
 
-POE::Component::Server::TCP is Copyright 2000-2009 by Rocco Caputo.
+POE::Component::Server::TCP is Copyright 2000-2013 by Rocco Caputo.
 All rights are reserved.  POE::Component::Server::TCP is free
 software, and it may be redistributed and/or modified under the same
 terms as Perl itself.
