@@ -3,7 +3,7 @@ package POE::Loop;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.280'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.370'; # NOTE - Should be #.### (three decimal places)
 
 1;
 
@@ -46,7 +46,7 @@ As POE::Kernel loads, it searches through %INC for event loop modules.
 POE::Kernel loads the most appropriate POE::Loop subclass for the
 event loop it finds.  The subclass slots its methods into POE::Kernel,
 completing the class at load time.  POE and POE::Kernel provide ways
-to state the desired event loop in case the autodetection makes a
+to state the desired event loop in case the auto-detection makes a
 mistake or the developer prefers to be explicit.  See
 L<POE::Kernel/"Using POE with Other Event Loops"> for instructions on
 how to actually use POE with other event loops, event loop naming
@@ -256,7 +256,7 @@ Watch for a given SIGNAL_NAME.  SIGNAL_NAME is the version found in
 "SIG" removed.
 
 POE::Loop::PerlSignals' implementation adds callbacks to %SIG except
-for CHLD/CLD, which begins a waitpid() pollng loop instead.
+for CHLD/CLD, which begins a waitpid() polling loop instead.
 
 As of this writing, all of the POE::Loop subclasses register their
 signal handlers through POE::Loop::PerlSignals.
@@ -315,7 +315,7 @@ Here's Gtk's:
           $self->_dispatch_event(
             $self, $self,
             EN_SIGNAL, ET_SIGNAL, [ 'UIDESTROY' ],
-            __FILE__, __LINE__, undef, time(), -__LINE__
+            __FILE__, __LINE__, undef, monotime(), -__LINE__
           );
         }
         return 0;
@@ -337,8 +337,8 @@ dispatched.
 
 The time management methods may accept NEXT_EVENT_TIME.  This is the
 time the next event will become due, in UNIX epoch time.
-NEXT_EVENT_TIME is a real number and may have subsecond accuracy.  It
-is the bridge's responsibility to convert this value intos something
+NEXT_EVENT_TIME is a real number and may have sub-second accuracy.  It
+is the bridge's responsibility to convert this value into something
 the underlying event loop requires.
 
 =head3 loop_resume_time_watcher NEXT_EVENT_TIME
@@ -346,7 +346,7 @@ the underlying event loop requires.
 Resume an already active time watcher.  It is used with
 loop_pause_time_watcher() to provide less expensive timer toggling for
 frequent use cases.  As mentioned above, NEXT_EVENT_TIME is in UNIX
-epoch time and may have subsecond accuracy.
+epoch time and may have sub-second accuracy.
 
 loop_resume_time_watcher() is used by bridges that set them watchers
 in the underlying event loop.  For example, POE::Loop::Gtk implements
@@ -517,7 +517,7 @@ versions.
 =head3 loop_resume_filehandle FILE_HANDLE, IO_MODE
 
 This is a lightweight form of loop_watch_filehandle().  It is used
-along with loop_pause_filehandle() to temporarily toggle a a watcher's
+along with loop_pause_filehandle() to temporarily toggle a watcher's
 state for a FILE_HANDLE in a particular IO_MODE.
 
 =head1 HOW POE FINDS EVENT LOOP BRIDGES
@@ -555,10 +555,14 @@ L<POE::Test::Loops> is POE's event loop tests released as a separate,
 reusable distribution.  POE::Loop authors are encouraged to use the
 tests for their own distributions.
 
+=for comment
 TODO - Link to CPAN for POE::Loop modules.
 
 =head1 BUGS
 
+None known.
+
+=for comment
 TODO - Link to POE bug queue.
 
 =head1 AUTHORS & LICENSING
