@@ -3,7 +3,7 @@ package POE::Session;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294'; # NOTE - Should be #.### (three decimal places)
+$VERSION = '1.370'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw(carp croak);
 use Errno;
@@ -11,6 +11,7 @@ use Errno;
 sub SE_NAMESPACE    () { 0 }
 sub SE_OPTIONS      () { 1 }
 sub SE_STATES       () { 2 }
+sub SE_ID           () { 3 }
 
 sub CREATE_ARGS     () { 'args' }
 sub CREATE_OPTIONS  () { 'options' }
@@ -426,7 +427,7 @@ sub _invoke_state {
       $! = exists &Errno::ENOSYS ? &Errno::ENOSYS : &Errno::EIO;
       if ($self->[SE_OPTIONS]->{+OPT_DEFAULT} and $state ne EN_SIGNAL) {
         my $loggable_self =
-          $POE::Kernel::poe_kernel->_data_alias_loggable($self);
+          $POE::Kernel::poe_kernel->_data_alias_loggable($self->ID);
         POE::Kernel::_warn(
           "a '$state' event was sent from $file at $line to $loggable_self ",
           "but $loggable_self has neither a handler for it ",
@@ -589,8 +590,13 @@ sub _register_state {
 # Return the session's ID.  This is a thunk into POE::Kernel, where
 # the session ID really lies.
 
+sub _set_id {
+  my ($self, $id) = @_;
+  $self->[SE_ID] = $id;
+}
+
 sub ID {
-  $POE::Kernel::poe_kernel->ID_session_to_id(shift);
+  return shift()->[SE_ID];
 }
 
 #------------------------------------------------------------------------------
@@ -907,7 +913,7 @@ methods in the same object.  C<$_[OBJECT]> helps this happen.
   }
 
 You may also use method inheritance.  Here we invoke
-$self->a_method(@_).  Since Perl's C<<->>> operator unshifts $self
+$self->a_method(@_).  Since Perl's C<< -> >> operator unshifts $self
 onto the beginning of @_, we must first shift a copy off to maintain
 POE's parameter offsets:
 
@@ -1012,7 +1018,7 @@ C<$_[SENDER]> is commonly used as a return address for responses.  It may
 also be compared against C<$_[KERNEL]> to verify that timers and other
 POE::Kernel-generated events were not spoofed.
 
-This C<echo_handler()> reponds to the sender with an "echo" event that
+This C<echo_handler()> responds to the sender with an "echo" event that
 contains all the parameters it received.  It avoids a feedback loop by
 ensuring the sender session and event (STATE) are not identical to the
 current ones.
@@ -1022,8 +1028,9 @@ current ones.
     $_[KERNEL]->post( $_[SENDER], "echo", @_[ARG0..$#_] );
   }
 
+=for comment
 TODO - Document which events should have $_[SENDER] == $_[KERNEL].
-Probably in POE::Kernel.
+Probably in POE::Kernel.>
 
 =head3 $_[CALLER_FILE], $_[CALLER_LINE] and $_[CALLER_STATE]
 
@@ -1033,6 +1040,7 @@ are fairly plain.  CALLER_STATE contains the name of the event that
 was being handled when the event was created, or when the event
 watcher that ultimately created the event was registered.
 
+=for comment
 TODO - Rename SENDER_FILE, SENDER_LINE, SENDER_STATE?
 
 =head3 @_[ARG0..ARG9] or @_[ARG0..$#_]
@@ -1131,28 +1139,30 @@ application code has not saved a copy of it.
 C<create()> accepts several named parameters, most of which are optional.
 Note however that the parameters are not part of a hashref.
 
-TODO - Is it time to bring new() back as a synonym for create()?
-TODO PG - NO!  IMHO ->new implies simply creating the object, and 
-TODO that you have to hold onto the object.  ->create implies other actions
-TODO are happening, and that you don't want to hold on to it.
+=for comment
+Is it time to bring new() back as a synonym for create()?
+PG - NO!  IMHO ->new implies simply creating the object, and
+that you have to hold onto the object.  ->create implies other actions
+are happening, and that you don't want to hold on to it.
 
+=for comment
 TODO - Provide forward-compatible "handler" options and methods as
 synonyms for the "state" versions currently supported?
-TODO PG - No, that's for 1.01
+PG - No, that's for 1.01
 
+=for comment
 TODO - Add a "class_handlers" as a synonym for "package_handlers"?
-TODO PG - Maybe. However, to many synonyms can be a pain for an API.  
+PG - Maybe. However, to many synonyms can be a pain for an API.
 
+=for comment
 TODO - The above TODOs may be summarized: "deprecate old language"?
-TODO PG - Oh, you are thinking of deprecating the old language... erm... no?
-
-
+PG - Oh, you are thinking of deprecating the old language... erm... no?
 
+=for comment
 TODO PG - I notice these =head3 are in alphabetical order.  I think
 TODO all the *_states options should be together.  Followed by heap, args,
 TODO options
 
-
 =head3 args => ARRAYREF
 
 The C<args> parameter accepts a reference to a list of parameters that
@@ -1251,7 +1261,7 @@ resolve to a different subroutine.
 
 POE::Session sessions support a small number of options, which may be
 initially set with the C<option> constructor parameter and changed at
-run time with the C<option()|/option> mehtod.
+run time with the C<option()|/option> method.
 
 C<option> takes a hashref with option =E<gt> value pairs:
 
@@ -1630,6 +1640,7 @@ ASSERT_STATES by defaulting the "default" option to true instead of
 false.  See the option() method earlier in this document for details
 about the "default" option.
 
+=for comment
 TODO - It's not much of an assertion if it only warns.
 
 =head1 SEE ALSO
@@ -1677,7 +1688,7 @@ Reduced to its essence:
   my %event_handlers;
   $event_handler{_start} = sub { \%event_handlers };
 
-Note also that a anonymous sub creates a closure on all lexical
+Note also that an anonymous sub creates a closure on all lexical
 variables in the scope it was defined in, even if it doesn't reference
 them.  $session is still being held in a circular reference here:
 
