@@ -2,12 +2,12 @@ package POE::Wheel::ReadWrite;
 
 use strict;
 
-use vars qw($VERSION);
-$VERSION = '1.294'; # NOTE - Should be #.### (three decimal places)
+use vars qw($VERSION @ISA);
+$VERSION = '1.370'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( croak carp );
 use POE qw(Wheel Driver::SysRW Filter::Line);
-use base qw(POE::Wheel);
+push @ISA, qw(POE::Wheel);
 
 # Offsets into $self.
 sub HANDLE_INPUT               () {  0 }
@@ -774,6 +774,7 @@ L<POE::Driver::SysRW>, which implements sysread() and syswrite(), is the
 default.  It's used in nearly all cases, so there's no point in
 specifying it.
 
+=for comment
 TODO - Example.
 
 =head4 Filter
@@ -783,6 +784,7 @@ input data and the serializer it uses to prepare data for writing.  It
 defaults to a new L<POE::Filter::Line> instance since many network
 protocols are line based.
 
+=for comment
 TODO - Example.
 
 =head4 InputFilter
@@ -790,6 +792,7 @@ TODO - Example.
 InputFilter and OutputFilter may be used to specify different filters
 for input and output.
 
+=for comment
 TODO - Example.
 
 =head4 OutputFilter
@@ -942,6 +945,7 @@ HighEvent and LowEvent are used along with HighMark and LowMark to
 control the flow of streamed output.  Please see L</HighEvent> for
 more information and examples.
 
+=for comment
 TODO - Example here.
 
 =head2 put RECORDS
@@ -971,6 +975,7 @@ generates the event.  So for example, stopping InputEvent will also
 stop reading from the filehandle.  L</pause_input> and
 L</resume_input> may be a better way to manage input events, however.
 
+=for comment
 TODO - Example.
 
 =head2 set_filter POE_FILTER
@@ -988,6 +993,7 @@ Switching filters can be tricky.  Please see the discussion of
 get_pending() in L<POE::Filter>.  Some filters may not support being
 dynamically loaded or unloaded.
 
+=for comment
 TODO - Example.
 
 =head2 set_input_filter POE_FILTER
@@ -996,6 +1002,7 @@ set_input_filter() changes a POE::Wheel::ReadWrite object's input
 filter while leaving the output filter unchanged.  This alters the way
 data is parsed without affecting how it's serialized for output.
 
+=for comment
 TODO - Example.
 
 =head2 set_output_filter POE_FILTER
@@ -1003,6 +1010,7 @@ TODO - Example.
 set_output_filter() changes how a POE::Wheel::ReadWrite object
 serializes its output but does not affect the way data is parsed.
 
+=for comment
 TODO - Example.
 
 =head2 get_input_filter
@@ -1014,6 +1022,7 @@ object may be introspected or altered via its own methods.
 There is no get_filter() method because there is no sane return value
 when input and output filters differ.
 
+=for comment
 TODO - Example.
 
 =head2 get_output_filter
@@ -1025,6 +1034,7 @@ object may be introspected or altered via its own methods.
 There is no get_filter() method because there is no sane return value
 when input and output filters differ.
 
+=for comment
 TODO - Example.
 
 =head2 set_high_mark HIGH_MARK_OCTETS
@@ -1038,6 +1048,7 @@ greater than HIGH_MARK_OCTETS.
 Both HighEvent and put() indicate that it's unsafe to continue writing
 when the output buffer expands to at least HIGH_MARK_OCTETS.
 
+=for comment
 TODO - Example.
 
 =head2 set_low_mark LOW_MARK_OCTETS
@@ -1050,6 +1061,7 @@ POE::Wheel::ReadWrite objects will emit a LowEvent when their output
 buffers shrink to LOW_MARK_OCTETS after having reached
 HIGH_MARK_OCTETS.
 
+=for comment
 TODO - Example.
 
 =head2 ID
@@ -1060,6 +1072,7 @@ stashed by its ID.  Events generated by the POE::Wheel::ReadWrite
 object will include the ID of the object, so that they may be matched
 back to their sources.
 
+=for comment
 TODO - Example.
 
 =head2 pause_input
@@ -1069,6 +1082,7 @@ watching for input, and thus stop emitting InputEvent events.  It's
 much more efficient than destroying the object outright, especially if
 an application intends to resume_input() later.
 
+=for comment
 TODO - Example.
 
 =head2 resume_input
@@ -1078,6 +1092,7 @@ back on.  It's used to resume watching for input, and thus resume
 sending InputEvent events.  pause_input() and resume_input() implement
 a form of input flow control, driven by the application itself.
 
+=for comment
 TODO - Example.
 
 =head2 get_input_handle
@@ -1088,6 +1103,7 @@ Manipulating filehandles that are managed by POE may cause nasty side
 effects, which may change from one POE release to the next.  Please
 use caution.
 
+=for comment
 TODO - Example.
 
 =head2 get_output_handle
@@ -1098,6 +1114,7 @@ Manipulating filehandles that are managed by POE may cause nasty side
 effects, which may change from one POE release to the next.  Please
 use caution.
 
+=for comment
 TODO - Example.
 
 =head2 shutdown_input
@@ -1110,6 +1127,7 @@ Occasionally, the POE::Wheel::ReadWrite object will stop monitoring
 its input filehandle for new data.  This occurs regardless of the
 filehandle type.
 
+=for comment
 TODO - Example.
 
 =head2 shutdown_output
@@ -1122,6 +1140,7 @@ Occasionally, the POE::Wheel::ReadWrite object will stop monitoring its
 output filehandle for new data. This occurs regardless of the
 filehandle type.
 
+=for comment
 TODO - Example.
 
 =head2 get_driver_out_octets
@@ -1130,6 +1149,7 @@ L<POE::Driver> objects contain output buffers that are flushed
 asynchronously.  get_driver_out_octets() returns the number of octets
 remaining in the wheel's driver's output buffer.
 
+=for comment
 TODO - Example.
 
 =head2 get_driver_out_messages
@@ -1142,6 +1162,7 @@ Stream-based drivers will simply return 1 if any data remains to be
 flushed.  This is because they operate with one potentially large
 message.
 
+=for comment
 TODO - Example.
 
 =head2 flush
@@ -1165,7 +1186,8 @@ It should be obvious by now that B<this method is experimental>.  Its
 behavior may change or it may disappear outright.  Please let us know
 whether it's useful.
 
-# TODO - Example?
+=for comment
+TODO - Example?
 
 =head1 SEE ALSO
 
